/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#ifndef BLOBFIELDINITIALIZER_H
#define BLOBFIELDINITIALIZER_H

#include <CompuCell3D/Steppable.h>
#include <CompuCell3D/Field3D/Dim3D.h>

namespace CompuCell3D {
  class Potts3D;
  class Dim3D;
   
  class BlobFieldInitializer : public Steppable {
  protected:
    Potts3D *potts;

    int gap;
    int width;
    int radius;
    Dim3D blobDim;
    bool cellSortInit;
    
  public:
    BlobFieldInitializer();
    BlobFieldInitializer(int gap, int width);
    virtual ~BlobFieldInitializer(){}
    void setPotts(Potts3D *potts) {this->potts = potts;}
    double distance(double, double, double, double, double, double);
    Dim3D getBlobDim(){return blobDim;}
    
    // SimObject interface
    virtual void init(Simulator *simulator);

    // Begin Steppable interface
    virtual void start();
    virtual void step(const unsigned int currentStep) {}
    virtual void finish() {}
    // End Steppable interface
    Dim3D getBlobDimensions(const Dim3D & dim,int size);
    void initializeCellTypesCellSort();
    
    // Begin XMLSerializable interface
    virtual void readXML(XMLPullParser &in);
    virtual void writeXML(XMLSerializer &out);
    // End XMLSerializable interface
  };
};
#endif
