/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#ifndef TRIPLEVIZDUMPER_H
#define TRIPLEVIZDUMPER_H

#include <string>

#include <CompuCell3D/Field3D/Field3D.h>
#include <CompuCell3D/Viz/VizProperties.h>
#include <CompuCell3D/Steppable.h>

namespace CompuCell3D {
  class FieldRenderer;

  /** 
   * Writes a Field3D of VizProperties in ogle format.
   *
   * See http://www.cora.nwra.com/Ogle/ for more information.
   *
   * Requires zlib.  See http://www.gzip.org/zlib/ for more information.
   */
  class TripleVizDumper : public Steppable {
    FieldRenderer *renderer;

    std::string filePrefix;
    Dim3D gridSize;
    
    // Target colors, for the solo visualization.
    unsigned int tred, tgreen, tblue, topacity;
  public:
    TripleVizDumper() : renderer(0) {}
    virtual ~TripleVizDumper() {}
    
    // Begin Steppable interface
    virtual void start() {step(0);}
    virtual void step(const unsigned int currentStep);
    virtual void finish() {}
    // End Steppable interface

    // Begin XMLSerializable interface
    virtual void readXML(XMLPullParser &in);
    virtual void writeXML(XMLSerializer &out);
    // End XMLSerializable interface
  };
};
#endif
