/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include "FieldRenderer.h"

#include <CompuCell3D/Potts3D/Potts3D.h>
#include <CompuCell3D/Simulator.h>
using namespace CompuCell3D;

#include <BasicUtils/BasicException.h>


void FieldRenderer::init(Simulator *simulator) {
  cellFieldG = simulator->getPotts()->getCellFieldG();
  SimObject::init(simulator);
}

void FieldRenderer::set(const Point3D &pt, const VizProperties value) {
  THROW("set() A field G cannot be changed through the FieldReneder interface!");
}

Dim3D FieldRenderer::getDim() const {
  ASSERT_OR_THROW("get() Cell field G not set!", cellFieldG);
  return cellFieldG->getDim();
}

bool FieldRenderer::isValid(const Point3D &pt) const {
  ASSERT_OR_THROW("get() Cell field not set!", cellFieldG);
  return cellFieldG->isValid(pt);
}
