/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#ifndef CHEMOTAXISDICTYENERGY_H
#define CHEMOTAXISDICTYENERGY_H

#include <CompuCell3D/Potts3D/EnergyFunction.h>

#include <XMLCereal/XMLSerializable.h>
#include <CompuCell3D/Potts3D/Cell.h>


#include <string>
#include <vector>


template <typename T>
class BasicClassAccessor;

namespace CompuCell3D {
  template <class T>
  class Field3D;
  template <class T>
  class Field3DImpl;

  class Potts3D;
  class Simulator;
  class SimpleClock;

  class ChemotaxisDictyEnergy : public EnergyFunction, public XMLSerializable {
    Field3DImpl<float> *field;
   
    Potts3D *potts;
    Simulator *sim;
    BasicClassAccessor<SimpleClock> *simpleClockAccessorPtr;
    
    double lambda;

    std::string chemicalFieldSource;
    std::string chemicalFieldName;
    bool chemotax;
    bool gotChemicalField;

    std::vector<unsigned char> nonChemotacticTypeVector;
    
  public:
    ChemotaxisDictyEnergy(Potts3D *potts,
		   const double threshold, const double lambda) :
      field(0), potts(potts), lambda(lambda),gotChemicalField(false) {}

    virtual ~ChemotaxisDictyEnergy() {}

    void setSimulatorPtr(Simulator * _sim){sim=_sim;}

    void setSimpleClockAccessorPtr(BasicClassAccessor<SimpleClock> *_simpleClockAccessorPtr){
      simpleClockAccessorPtr=_simpleClockAccessorPtr;
    }
    
    double getConcentration(const Point3D &pt);

    double getLambda() {return lambda;}
    

    Field3D<float>* getField() {return field;}
    void setField(Field3DImpl<float>* _field ) {field=_field;}

    
   void initializeField();

    virtual double localEnergy(const Point3D &pt);
    virtual double changeEnergy(const Point3D &pt, const CellG *newCell,
                                const CellG *oldCell);

    // Begin XMLSerializable interface
    virtual void readXML(XMLPullParser &in);
    virtual void writeXML(XMLSerializer &out);
    // End XMLSerializable interface
  };
};
#endif
