/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR1 PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#ifndef SURFACELOCALENERGY_H
#define SURFACELOCALENERGY_H

#include <CompuCell3D/Potts3D/EnergyFunction.h>

#include <XMLCereal/XMLSerializable.h>



namespace CompuCell3D {

  class CellSurface;

  template <class T> class Field3D;
  class Point3D;

  /** 
   * Calculates surface energy based on a target surface and
   * lambda surface.
   */
  class SurfaceLocalEnergy: public EnergyFunction, public virtual XMLSerializable {
    
    Field3D<CellG *> *cellFieldG;
    
       
    unsigned int targetSurface;
    double lambdaSurface;

  public:
    SurfaceLocalEnergy(Field3D<CellG *> *_cellFieldG) :
          cellFieldG(_cellFieldG), targetSurface(0),
      lambdaSurface(0) {}
    

      
    double diffLocalEnergy(const CellG *cell, int diff);

    virtual double localEnergy(const Point3D &pt);
    virtual double changeEnergy(const Point3D &pt, const CellG *newCell,
				const CellG *oldCell);

				
    // Begin XMLSerializable interface
    virtual void readXML(XMLPullParser &in);
    virtual void writeXML(XMLSerializer &out);
    // End XMLSerializable interface
  };
};
#endif
