/*************************************************************************
 *    CompuCell - A software framework for multimodel simulations of     *
 * biocomplexity problems Copyright (C) 2003 University of Notre Dame,   *
 *                             Indiana                                   *
 *                                                                       *
 * This program is free software; IF YOU AGREE TO CITE USE OF CompuCell  *
 *  IN ALL RELATED RESEARCH PUBLICATIONS according to the terms of the   *
 *  CompuCell GNU General Public License RIDER you can redistribute it   *
 * and/or modify it under the terms of the GNU General Public License as *
 *  published by the Free Software Foundation; either version 2 of the   *
 *         License, or (at your option) any later version.               *
 *                                                                       *
 * This program is distributed in the hope that it will be useful, but   *
 *      WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *             General Public License for more details.                  *
 *                                                                       *
 *  You should have received a copy of the GNU General Public License    *
 *     along with this program; if not, write to the Free Software       *
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *************************************************************************/

#include "VolumeFlexPlugin.h"

#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
using namespace CompuCell3D;

#include <XMLCereal/XMLPullParser.h>
#include <XMLCereal/XMLSerializer.h>


#include <iostream>
using namespace std;

VolumeFlexPlugin::VolumeFlexPlugin() : potts(0), deadCellG(0) {}

VolumeFlexPlugin::~VolumeFlexPlugin() {}

void VolumeFlexPlugin::init(Simulator *simulator) {
  this->simulator=simulator; 
  potts = simulator->getPotts();  
  potts->registerEnergyFunction(&volumeEnergy);
  potts->registerCellGChangeWatcher(this);
  potts->registerStepper(this);

  //volumeEnergy->setSimualtor(simulator);
  
}

void VolumeFlexPlugin::extraInit(Simulator *simulator){
   volumeEnergy.initTypeId(potts);
}


void VolumeFlexPlugin::field3DChange(const Point3D &pt, CellG *newCell,
				 CellG *oldCell) {

      
   if (newCell){
      newCell->volume++;
   }
   
   if (oldCell){
      
      if((--oldCell->volume) == 0)
         deadCellG = oldCell;    
   } 
}


void VolumeFlexPlugin::step() {
   
  if (deadCellG) {
    potts->destroyCellG(deadCellG);
    deadCellG = 0;
  }
}

void VolumeFlexPlugin::readXML(XMLPullParser &in) {
  volumeEnergy.readXML(in);
}

void VolumeFlexPlugin::writeXML(XMLSerializer &out) {
  volumeEnergy.writeXML(out);
}
